<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00003000ParamFatEscopoFactory {

    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function getParametrosDeFaturamentoDoEscopo(array $params) {
        $sql = "SELECT NRSEQFATUESCOPO,
                       NRTIPOORCAMENTO,
                       CDPRODFATURA,
                       CDFILIALFATURA,
                       NRGRUPOFATURA,
                       NRPARCFATURA,
                       IDFATURACONTR,
                       CDMODPGTO,
                       IDQTDIASCOMPCON,
                       NRQTDIASCOMPCON,
                       IDTPCOMPETCONT,
                       NRDIACOMPETCONT,
                       IDTPFATUINCOMP,
                       IDTPDTEMISSNF,
                       NRDIADTEMISSNF,
                       IDTPDTVENC,
                       NRDIATPDTVENC,
                       IDSOMAFATUCONT,
                       IDSOMAITGRUPFAT,
                       IDGERANFPARC,
                       IDBLOQVRMEDIC,
                       IDGERARATEIO,
                       NRORG
                  FROM FCT_PARAMFATESCOPO
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca quais dos escopos passados como parâmetro não possuem informações de faturamento
    **/
    public function getEscoposSemParametrosFaturamento(array $params, $nrtipoorcamento) {
        $sql = "SELECT NMTIPOORCAMENTO,
                       NRTIPOORCAMENTO,
                       CDSERVORCA,
                       IDTPFATURA,
                       CDTIPOOPERFATURA,
                       CDSERIENF,
                       CDEMPRESA,
                       CDINSCESTA,
                       CDINSCMUNI,
                       CDPRODFATURA,
                       CDFILIALFATURA,
                       NRPARCFATURA,
                       DECODE(IDTPFATURA, 'P', QTPARCELASFATU, NULL) AS QTPARCELASFATU,
                       DECODE(IDTPFATURA, 'P', QTDIASAPURACAO, NULL) AS QTDIASAPURACAO,
                       DECODE(IDTPFATURA, 'P', NRDIACALCVENC, NULL) AS NRDIACALCVENC,
                       DECODE(IDTPFATURA, 'P', QTDIASFATUVENC, NULL) AS QTDIASFATUVENC,
                       DECODE(IDTPFATURA, 'P', IDTPCALCVENC, NULL) AS IDTPCALCVENC,
                       DECODE(IDTPFATURA, 'P', IDMESCALCVENC, NULL) AS IDMESCALCVENC,
                       DECODE(IDTPFATURA, 'P', 0, (SELECT COUNT(CIT.NRPARAMFATITESCSERV)
                                                     FROM FCT_PARAMFATITESCSERV CIT
                                                    INNER JOIN FCT_ITESCOPOPROPOSTA ITESC
                                                       ON CIT.NRITESCOPOPROPOSTA = ITESC.NRITESCOPOPROPOSTA
                                                      AND CIT.NRORG = ITESC.NRORG
                                                    WHERE ITESC.NRTIPOORCAMENTO = TBL.NRTIPOORCAMENTO
                                                      AND ITESC.CDSERVORCA = TBL.CDSERVORCA)) AS QTCOMPETENCIA
                  FROM (
                    SELECT ESC.NMTIPOORCAMENTO,
                           ITESC.NRTIPOORCAMENTO,
                           ITESC.CDSERVORCA,
                           NVL(PFIT.IDTPFATURA, PFESC.IDTPFATURA) AS IDTPFATURA,
                           NVL(PFIT.CDTIPOOPERFATURA, PFESC.CDTIPOOPERFATURA) AS CDTIPOOPERFATURA,
                           NVL(PFIT.CDSERIENF, PFESC.CDSERIENF) AS CDSERIENF,
                           NVL(PFIT.CDEMPRESA, PFESC.CDEMPRESA) AS CDEMPRESA,
                           NVL(PFIT.CDINSCESTA, PFESC.CDINSCESTA) AS CDINSCESTA,
                           NVL(PFIT.CDINSCMUNI, PFESC.CDINSCMUNI) AS CDINSCMUNI,
                           NVL(PFIT.CDPRODFATURA, NVL(PFESC.CDPRODFATURA, SERV.CDPRODFATSERV)) AS CDPRODFATURA,
                           NVL(PFIT.CDFILIALFATURA, PFESC.CDFILIALFATURA) AS CDFILIALFATURA,
                           NVL(PFIT.NRPARCFATURA, PFESC.NRPARCFATURA) AS NRPARCFATURA,
                           NVL(PFIT.QTPARCELASFATU, PFESC.QTPARCELASFATU) AS QTPARCELASFATU,
                           NVL(PFIT.QTDIASAPURACAO, PFESC.QTDIASAPURACAO) AS QTDIASAPURACAO,
                           NVL(PFIT.NRDIACALCVENC, PFESC.NRDIACALCVENC) AS NRDIACALCVENC,
                           NVL(PFIT.QTDIASFATUVENC, PFESC.QTDIASFATUVENC) AS QTDIASFATUVENC,
                           NVL(PFIT.IDTPCALCVENC, PFESC.IDTPCALCVENC) AS IDTPCALCVENC,
                           NVL(PFIT.IDMESCALCVENC, PFESC.IDMESCALCVENC) AS IDMESCALCVENC
                      FROM FCT_ITESCOPOPROPOSTA ITESC
                     INNER JOIN FCT_SERVICOORCA SERV
                        ON ITESC.CDSERVORCA = SERV.CDSERVORCA
                       AND ITESC.NRORG      = SERV.NRORG
                      LEFT JOIN FCT_PARAMFATITESC PFIT
                        ON ITESC.NRTIPOORCAMENTO    = PFIT.NRTIPOORCAMENTO
                       AND ITESC.NRITESCOPOPROPOSTA = PFIT.NRITESCOPOPROPOSTA
                       AND ITESC.NRORG = PFIT.NRORG
                      LEFT JOIN FCT_CAPAPARAMFATESC PFESC
                        ON ITESC.NRTIPOORCAMENTO = PFESC.NRTIPOORCAMENTO
                       AND ITESC.NRORG           = PFESC.NRORG
                      JOIN FCT_TIPOORCAMENTO ESC
                        ON ITESC.NRTIPOORCAMENTO = ESC.NRTIPOORCAMENTO
                       AND ITESC.NRORG           = ESC.NRORG
                     WHERE ITESC.NRTIPOORCAMENTO IN ($nrtipoorcamento)
                       AND ITESC.NRORG           = :NRORG
                ) TBL
                ORDER BY NMTIPOORCAMENTO,
                         NRTIPOORCAMENTO,
                         IDTPFATURA,
                         CDTIPOOPERFATURA,
                         CDSERIENF,
                         CDEMPRESA,
                         CDINSCESTA,
                         CDINSCMUNI,
                         CDFILIALFATURA,
                         NRPARCFATURA,
                         QTPARCELASFATU,
                         QTDIASAPURACAO,
                         NRDIACALCVENC,
                         QTDIASFATUVENC,
                         IDTPCALCVENC,
                         IDMESCALCVENC,
                         QTCOMPETENCIA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Busca os dados da competência de um escopo
    **/
    public function getCompetenciaItemNegociacao(array $params) {
        $sql = "SELECT NRDIACOMPETINI,
                       NRDIACOMPETFIN,
                       IDMESCOMPETINI,
                       IDMESCOMPETFIN,
                       QTDIASAPURACAO,
                       IDTPCALCVENC,
                       NRDIACALCVENC,
                       IDMESCALCVENC,
                       QTDIASFATUVENC
                  FROM (
                    SELECT PFIT.NRDIACOMPETINI,
                           PFIT.NRDIACOMPETFIN,
                           PFIT.IDMESCOMPETINI,
                           PFIT.IDMESCOMPETFIN,
                           PFIT.QTDIASAPURACAO,
                           PFIT.IDTPCALCVENC,
                           PFIT.NRDIACALCVENC,
                           PFIT.IDMESCALCVENC,
                           PFIT.QTDIASFATUVENC
                      FROM FCT_PARAMFATITESCSERV PFIT
                     INNER JOIN FCT_ITESCOPOPROPOSTA IT
                        ON PFIT.NRITESCOPOPROPOSTA = IT.NRITESCOPOPROPOSTA
                       AND PFIT.NRORG              = IT.NRORG
                     WHERE IT.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                       AND IT.NRORG           = :NRORG
                     UNION ALL
                    SELECT NRDIACOMPETINI,
                           NRDIACOMPETFIN,
                           IDMESCOMPETINI,
                           IDMESCOMPETFIN,
                           QTDIASAPURACAO,
                           IDTPCALCVENC,
                           NRDIACALCVENC,
                           IDMESCALCVENC,
                           QTDIASFATUVENC
                      FROM FCT_PARAMFATESC
                     WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                       AND NRORG           = :NRORG
                       AND NOT EXISTS (SELECT 1
                                         FROM FCT_PARAMFATITESCSERV
                                        WHERE NRITESCOPOPROPOSTA IN (SELECT NRITESCOPOPROPOSTA
                                                                      FROM FCT_ITESCOPOPROPOSTA
                                                                     WHERE NRTIPOORCAMENTO IN :NRTIPOORCAMENTO))
                )
                ORDER BY IDMESCOMPETINI, NRDIACOMPETINI";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
     /**
     * A função "validaPercentual" valida o percentual para inserção na página "índices Financeiros Padrões"
     */
    
    public function validaPercentual(array $params){
        $sql = "SELECT SUM(NRPERCENTINDICE) AS TOTPERCENT 
                  FROM FCT_INDICESPADRESCOPO
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG           = :NRORG
                   AND (CDINDICE <> :CDINDICE OR :CDINDICE IS NULL)";
                 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o valor do campo "Tipo de cálculo para parcelas incompletas"
     */
    public function getTipoCalculoPeriodosIncompletos(array $params) {
        $sql = "SELECT IDTPCALCPARCINC
                  FROM FCT_CAPAPARAMFATESC
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}

?>